import logging

from pytorch_lightning.callbacks import ModelCheckpoint as _ModelCheckpoint

__all__ = ["ModelCheckpoint"]

logger = logging.getLogger(__name__)


class ModelCheckpoint(_ModelCheckpoint):

    r"""
    ModelCheckpoint same as pl, plus logging and train_epoch_end callback.

    1. log output when save checkpoint.
    2. add train_epoch_end callback.

    """

    def __init__(self, *args, **kwargs):
        self.on_epoch = kwargs.pop("on_epoch", False)
        super(ModelCheckpoint, self).__init__(*args, **kwargs)

    def on_train_epoch_end(self, trainer, pl_module, outputs):
        """
        save checkpoint every epoch end.
        """
        if self.on_epoch:
            self.save_checkpoint(trainer, pl_module)

    def save_checkpoint(self, trainer, pl_module):
        super(ModelCheckpoint, self).save_checkpoint(trainer, pl_module)
        epoch = trainer.current_epoch
        global_step = trainer.global_step
        monitor_candidates = self._monitor_candidates(trainer)
        filepath = self.format_checkpoint_name(
            epoch, global_step, monitor_candidates
        )
        if self._fs.exists(filepath):
            logger.info(f"Save checkpoint to {filepath}")



import logging
import os
from contextlib import contextmanager
from typing import Dict

from pl_extension.utilities.env import HDFS_CACHE

import mmcv
from iopath.common.file_io import HTTPURLHandler, PathManagerFactory

__all__ = ["load_file", "open_file", "path_manager"]


logger = logging.getLogger(__name__)


def load_file(file_path: str) -> Dict:
    """
    load file from .py/.yaml/.json/.yml/.pkl
    """
    file_ext = os.path.splitext(file_path)[1]
    assert file_ext in [
        ".py",
        ".yaml",
        "yml",
        "json",
        "pkl",
    ], f" Bad file ext: {file_path}"

    if file_ext == ".py":
        content = mmcv.Config.fromfile(file_path)
        content = dict(content)
    else:
        content = mmcv.load(file_path)
    assert isinstance(content, dict)
    return content


@contextmanager
def open_file(path: str, mode: str = None, cachedir: str = HDFS_CACHE):
    """
    Open file in hdfs, support local cache.
    """
    tmp_path = None
    if "hdfs://" in path:
        tmp_path = path.replace("/", "_").replace(":", "_")
        if cachedir:
            if not os.path.exists(cachedir):
                os.makedirs(cachedir)
            tmp_path = os.path.join(cachedir, tmp_path)
            if not os.path.exists(tmp_path):
                copy_command = "hdfs dfs -get %s %s" % (path, tmp_path)
                logger.info(copy_command)
                os.system(copy_command)
            else:
                logger.info("use cache %s" % tmp_path)
        else:
            tmp_path = "%s_%s" % (os.path.basename(path), str(os.getpid()))
            if os.path.exists(tmp_path):
                os.remove(tmp_path)
            copy_command = "hdfs dfs -get %s %s" % (path, tmp_path)
            logger.info(copy_command)
            os.system(copy_command)
        r = open(tmp_path, mode)
    else:
        r = open(path, mode)
    try:
        yield r
    finally:
        r.close()
        if tmp_path is not None and cachedir is None:
            os.remove(tmp_path)


path_manager = PathManagerFactory.get(defaults_setup=True)

path_manager.register_handler(HTTPURLHandler())



"""
pl trainer io.
"""

from pl_extension.callbacks import ModelCheckpoint

import pytorch_lightning as pl

__all__ = ["get_checkpoint_dirpath"]


def get_checkpoint_dirpath(trainer: pl.Trainer) -> str:
    """
    return checkpoint dirpath.
    """
    ck_callbacks = [
        c for c in trainer.callbacks if isinstance(c, ModelCheckpoint)
    ]
    assert len(ck_callbacks) > 0
    return ck_callbacks[0].dirpath



"""
Below are logger setting.
They are mainly adopted from
https://github.com/facebookresearch/detectron2/blob/master/detectron2/utils/logger.py   # noqa
"""
import atexit
import functools
import logging
import os
import sys

from termcolor import colored

from .file_io import path_manager as pm

try:
    from pint_horizon.aidi import running_in_cluster
except ImportError:
    running_in_cluster = None


__all__ = ["setup_logger"]


class _ColorfulFormatter(logging.Formatter):
    def __init__(self, *args, **kwargs):
        self._root_name = kwargs.pop("root_name") + "."
        self._abbrev_name = kwargs.pop("abbrev_name", "")
        if len(self._abbrev_name):
            self._abbrev_name = self._abbrev_name + "."
        super(_ColorfulFormatter, self).__init__(*args, **kwargs)

    def formatMessage(self, record):
        record.name = record.name.replace(self._root_name, self._abbrev_name)
        log = super(_ColorfulFormatter, self).formatMessage(record)
        if record.levelno == logging.WARNING:
            prefix = colored("WARNING", "red", attrs=["blink"])
        elif (
            record.levelno == logging.ERROR
            or record.levelno == logging.CRITICAL
        ):
            prefix = colored("ERROR", "red", attrs=["blink", "underline"])
        else:
            return log
        return prefix + " " + log


# so that calling setup_logger multiple times won't add many handlers
@functools.lru_cache()
def setup_logger(
    output=None,
    distributed_rank=0,
    *,
    color=None,
    name="pl-extension",
    abbrev_name=None
):
    """
    Initialize the detectron2 logger and set its verbosity level to "DEBUG".

    Args:
        output (str): a file name or a directory to save log. If None,
            will not save log file.
            If ends with ".txt" or ".log", assumed to be a file name.
            Otherwise, logs will be saved to `output/log.txt`.
        name (str): the root module name of this logger
        abbrev_name (str): an abbreviation of the module, to avoid long names
            in logs. Set to "" to not log the root module in logs.
            By default, will abbreviate "detectron2" to "d2" and leave other
            modules unchanged.

    Returns:
        logging.Logger: a logger
    """
    logger = logging.getLogger(name)
    logger.setLevel(logging.DEBUG)
    logger.propagate = False

    if not color:
        color = not running_in_cluster() if running_in_cluster else True

    if abbrev_name is None:
        abbrev_name = "pl" if name == "pl-extension" else name

    plain_formatter = logging.Formatter(
        "[%(asctime)s] %(name)s %(levelname)s: %(message)s",
        datefmt="%m/%d %H:%M:%S",
    )
    # stdout logging: master only
    if distributed_rank == 0:
        ch = logging.StreamHandler(stream=sys.stdout)
        ch.setLevel(logging.DEBUG)
        if color:
            formatter = _ColorfulFormatter(
                colored("[%(asctime)s %(name)s]: ", "green") + "%(message)s",
                datefmt="%m/%d %H:%M:%S",
                root_name=name,
                abbrev_name=str(abbrev_name),
            )
        else:
            formatter = plain_formatter
        ch.setFormatter(formatter)
        logger.addHandler(ch)

    # file logging: all workers
    if output is not None:
        if output.endswith(".txt") or output.endswith(".log"):
            filename = output
        else:
            filename = os.path.join(output, "log.txt")
        if distributed_rank > 0:
            filename = filename + ".rank{}".format(distributed_rank)

        pm.mkdirs(os.path.dirname(filename))

        fh = logging.StreamHandler(_cached_log_stream(filename))
        fh.setLevel(logging.DEBUG)
        fh.setFormatter(plain_formatter)
        logger.addHandler(fh)

    return logger


# cache the opened file object, so that different calls to `setup_logger`
# with the same file name can safely write to the same file.
@functools.lru_cache(maxsize=None)
def _cached_log_stream(filename):
    io = pm.open(filename, "a")
    atexit.register(io.close)
    return io



import os
from typing import List

from pl_extension.utilities.logger import setup_logger
from pl_extension.utilities.rand import time_string

from pytorch_lightning.loggers.base import LightningLoggerBase
from pytorch_lightning.utilities import rank_zero_only

__all__ = ["LoggingLogger"]


class LoggingLogger(LightningLoggerBase):

    """
    Logging logger.

    Args:
        logdir: local logs save path.
        prefix: logfile name prefix, default is 'pl'.
        skip_metrics: skip certain metric, useful for iter-wise metrics.

    Example::

        from pl_extension.loggers import LoggingLogger
        logging_logger = LoggingLogger(logdir='logs', prefix='pl_extension')
        trainer = Trainer(logger=[logging_logger])

    """

    def __init__(
        self,
        logdir: str = ".pl_extension_logs",
        *,
        prefix: str = "pl",
        skip_metrics: List = [],
    ):
        super().__init__()
        if not os.path.exists(logdir):
            os.makedirs(logdir)
        logfile = os.path.join(logdir, f"{prefix}-{time_string()}.log")
        self._experiment = setup_logger(logfile, name=prefix)
        self._skip_metrics = skip_metrics

    def __getattr__(self, name):
        if name == "logger":
            self.experiment.warning(
                "xxx.logger is deprecated, " "please use xxx.experiment instead"
            )
            return self.experiment

    @property
    @rank_zero_only
    def experiment(self):
        return self._experiment

    @rank_zero_only
    def info(self, *args, **kwargs):
        self._experiment.info(*args, **kwargs)

    @rank_zero_only
    def warning(self, *args, **kwargs):
        self._experiment.warning(*args, **kwargs)

    @rank_zero_only
    def error(self, *args, **kwargs):
        self._experiment.error(*args, **kwargs)

    @rank_zero_only
    def log_metrics(self, metrics, step):
        # skip metric in list
        metrics = {
            k: metrics[k] for k in metrics if k not in self._skip_metrics
        }
        _str = ""
        if "epoch" in metrics:
            _str += "Epoch[%d] " % metrics.pop("epoch")
        if "iter" in metrics:
            _str += "Iter[%d] " % metrics.pop("iter")
        else:
            _str += "Step[%d] " % step
        if "speed" in metrics:
            _str += "Speed: %.2f samples/sec, " % metrics.pop("speed")
        for k in metrics:
            if isinstance(metrics[k], int):
                _format = "%s=%d, "
            elif isinstance(metrics[k], float):
                if k == "lr":
                    _format = "%s=%.6f, "
                else:
                    _format = "%s=%.4f, "
            else:
                raise ValueError(f"Unknown value type: {type(metrics[k])}")
            _str += _format % (k, metrics[k])
        if _str:
            _str = _str[:-2]
            self.experiment.info(_str)

    @rank_zero_only
    def log_hyperparams(self, params):
        pass

    @property
    def name(self):
        return "pint-logger"

    @property
    def version(self):
        pass



# -*- coding: utf-8 -*-

import io
import re

from setuptools import setup, find_packages

with open("README.md", "r") as f:
    readme = f.read()

with open("LICENSE", "r") as f:
    license = f.read()

with open("requirements/build.txt", "r") as f:
    requires = []
    for line in f:
        line = line.strip()
        if not line.startswith("#"):
            requires.append(line)

with io.open("pl_extension/__init__.py", "rt", encoding="utf8") as f:
    version = re.search(r"__version__ = \"(.*?)\"", f.read()).group(1)

setup(
    name="pl-extension",
    version=version,
    description="extension for pytorch-lightning",
    long_description=readme,
    author="duinodu",
    author_email="472365351@qq.com",
    url="https://github.com/duinodu/pl-extension",
    project_urls={
        "Bug Tracker": "https://github.com/duinodu/pl-extension/issues",
    },
    license=license,
    zip_safe=False,
    include_package_data=True,
    packages=find_packages(exclude=['tests', 'tests/*', 'docs']),
    python_requires=">=3.6",
    install_requires=requires,
)



import logging

from pytorch_lightning.callbacks.base import Callback

__all__ = ["Speed"]

logger = logging.getLogger(__name__)


class Speed(Callback):

    r"""
    Training speed callback, require 'simple' or 'advanced' profiler.

    """

    def on_train_batch_end(
        self, trainer, pl_module, outputs, batch, batch_idx, dataloader_idx
    ):
        legacy_metrics = (
            trainer.logger_connector.cached_results.legacy_batch_log_metrics
        )
        legacy_metrics["iter"] = trainer.global_step
        legacy_metrics["epoch"] = trainer.current_epoch
        if not self.__has_profiler(trainer):
            # if not profiler provided, skip speed and batch_time.
            return
        # get training one batch time
        run_training_batch_time = trainer.profiler.recorded_durations[
            "run_training_batch"
        ][-1]
        if hasattr(trainer.datamodule, "batch_size"):
            total_batch_size = (
                trainer.datamodule.batch_size * trainer.world_size
            )
            legacy_metrics["speed"] = (
                1.0 * total_batch_size / run_training_batch_time
            )
        else:
            legacy_metrics["batch_time"] = run_training_batch_time

    def on_train_epoch_end(self, trainer, pl_module, *args, **kwargs):
        if not self.__has_profiler(trainer):
            return
        run_training_epoch_time = trainer.profiler.recorded_durations[
            "run_training_epoch"
        ]
        if len(run_training_epoch_time) > 0 and hasattr(
            trainer.logger, "log_metrics"
        ):
            epoch_time = {"epoch_time": run_training_epoch_time[-1]}
            trainer.logger.log_metrics(epoch_time, step=trainer.current_epoch)

    def __has_profiler(self, trainer):
        return hasattr(trainer.profiler, "recorded_durations")
